
/* Chrysalide - Outil d'analyse de fichiers binaires
 * executable-int.h - prototypes de code utile aux formats d'exécutables
 *
 * Copyright (C) 2009-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _FORMAT_EXECUTABLE_INT_H
#define _FORMAT_EXECUTABLE_INT_H


#include "executable.h"


#include "format-int.h"



/* Indique le type d'architecture visée par le format. */
typedef const char * (* get_target_machine_fc) (const GExeFormat *);

/* Etend la définition des portions au sein d'un binaire. */
typedef void (* refine_portions_fc) (const GExeFormat *, GPortionLayer *);

/* Fournit l'emplacement correspondant à une position physique. */
typedef bool (* translate_phys_fc) (const GExeFormat *, phys_t, vmpa2t *);

/* Fournit l'emplacement correspondant à une adresse virtuelle. */
typedef bool (* translate_virt_fc) (const GExeFormat *, virt_t, vmpa2t *);

/* Fournit l'emplacement d'une section donnée. */
typedef bool (* get_range_by_name_fc) (const GExeFormat *, const char *, mrange_t *);



/* Format d'exécutable générique (instance) */
struct _GExeFormat
{
    GBinFormat parent;                      /* A laisser en premier        */

    GDbgFormat **debugs;                    /* Informations de débogage    */
    size_t debugs_count;                    /* Nombre de ces informations  */

    GPortionLayer *layers;                  /* Couches de morceaux binaires*/

};

/* Format d'exécutable générique (classe) */
struct _GExeFormatClass
{
    GBinFormatClass parent;                 /* A laisser en premier        */

    get_target_machine_fc get_machine;      /* Architecture ciblée         */
    refine_portions_fc refine_portions;     /* Décrit les portions binaires*/

    translate_phys_fc translate_phys;       /* Correspondance phys -> vmpa */
    translate_virt_fc translate_virt;       /* Correspondance virt -> vmpa */

    get_range_by_name_fc get_range_by_name; /* Emplacement de sections     */     

};


/* Fournit l'emplacement correspondant à une position physique. */
bool g_exe_format_without_virt_translate_offset_into_vmpa(const GExeFormat *, phys_t, vmpa2t *);

/* Fournit l'emplacement correspondant à une adresse virtuelle. */
bool g_exe_format_without_virt_translate_address_into_vmpa(const GExeFormat *, virt_t, vmpa2t *);



#endif  /* _FORMAT_EXECUTABLE_INT_H */
